/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */
package org.apache.beehive.controls.runtime.bean;

import java.lang.reflect.Method;

/**
 *  The InvokeListener interface can be implemented by contextual services or helper
 *  classes associated with a ControlBean that want pre/post hook notifications of
 *  invocations occuring on a ControlBean.
 *
 *  Hooking is "read only".  An InvokeListener cannot modify the invoked method,
 *  arguments, return value or thrown exceptions in any way.
 */
public interface InvokeListener extends java.util.EventListener
{
    /**
     * Called just prior to invoking an operation or callback event on a control.
     */
    public void preInvoke(Method m, Object [] args);

    /**
     * Called just after inovcation of an operation or callback event on a control
     */
    public void postInvoke(Object retval, Throwable t);
} 
